# -*- coding: utf-8 -*-
import warnings


class Table(object):
    """
    Basic class to print data in a table format.

    Prints a table with a provided headers and data. The number of header titles
        establishes the number of columns in the table.

    Attributes:
        table_data: An iterable containing the data to print in the table.
                    By default an empty list
        headers: Title of each column in the table. By default an empty list.
    """

    def __init__(self, table_data=[], headers=[]):
        self._table_data = table_data
        self._headers = list(map(str, headers))
        self._num_columns = len(headers)
        self._colspace = 3
        self._calc_columns_max_lenght()

    def __getitem__(self, i):
        return self._table_data[i * self._num_columns:i * self._num_columns + self._num_columns]

    def __str__(self):
        self.print_table()

    def __repr__(self):
        print('{\n\t"headers":{},\n\t"data":{},\n\n"columns_lenght":{}\n}'.format(self.headers,
                                                                                  self.table_data,
                                                                                  self._column_max))

    def _calc_columns_max_lenght(self):
        """
        Calculates max lenght of each column based on data in `_table_data` and `_headers`
        """
        if self._num_columns == 0:
            return

        column_max = []
        for head in self._headers:
            column_max.append(len(head))

        for i, data in enumerate(self._table_data):
            if len(str(data)) > column_max[i % self._num_columns]:
                column_max[i % self._num_columns] = len(str(data))

        self._column_max = column_max

    def _update_columns_max_lenght(self, data_list, start_index=0):
        """
        Updates max lenght of columns from a data list.
        """
        if self._num_columns == 0:
            return

        for i, data in enumerate(data_list, start=start_index):
            if len(str(data)) > self._column_max[i % self._num_columns]:
                self._column_max[i % self._num_columns] = len(str(data))

    def _print_hsep(self, char='_', borders=' '):
        """
        Print horizontal separator
        """
        print('{b}{l}{b}'.format(b=borders, l=char*(sum(self._column_max) + (self._num_columns + 1) * self._colspace)))

    def print_table(self):
        """
        Prints a table with the data.
        """

        if not self._num_columns:
            warnings.warn("Unable to calculate the number of columns, you need"
                          "to provide a 'headers' list to generate table layout."
                          "Use headers attribute", UserWarning)
            return

        # Print init
        col_space = ' '*self._colspace

        # Print separator line
        self._print_hsep()

        # Print headers
        headers_line = '|{s}{d:<{l}}{s}'.format(d=self._headers[0], l=self._column_max[0], s=col_space)
        for (col_lenght, header) in zip(self._column_max[1:], self._headers[1:]):
            headers_line += '{d:>{l}}{s}'.format(l=col_lenght, d=header, s=col_space)

        headers_line += '|'
        print(headers_line)
        self._print_hsep('-', '|')

        # Print lines
        for i, data in enumerate(self._table_data):
            if i % self._num_columns == 0:
                data_line = '|{s}{d:<{l}}{s}'.format(d=data, l=self._column_max[i % self._num_columns], s=col_space)

            else:
                data_line += '{d:>{l}}{s}'.format(d=data, l=self._column_max[i % self._num_columns], s=col_space)

            if i % self._num_columns == self._num_columns - 1:
                data_line += '|'
                print(data_line)

        self._print_hsep(borders='|')

    def add_data(self, data):
        """
        Add more data to the table.

        Args:
            data (list): List of data to add to the `Table`
        """
        current_index = len(self._table_data)
        self._update_columns_max_lenght(data, current_index)
        self._table_data.extend(data)

    @property
    def headers(self):
        """
        Headers of the table. If changed the number of columns will be recalculated.
        """
        return self._headers

    @headers.setter
    def headers(self, new_headers):
        self._headers = new_headers
        self._num_columns = len(new_headers)
        self._calc_columns_max_lenght()

    @property
    def table_data(self):
        """
        Change the data list to in the `Table`.
        """
        return self._table_data

    @table_data.setter
    def table_data(self, tab_data):
        self._table_data
        self._calc_columns_max_lenght()



import json
from tablat import Table
import unittest
from pathlib import Path


class TestTable(unittest.TestCase):

    _input_folder = Path(__file__).parent / 'input'
    _data_dict = {}
    _expected_data = []
    _target = None

    def test_table_init(self):
        data_obj = self.get_data()
        expected_data = self.get_test_expected()['test_table_init']

        table = Table(data_obj['data'], data_obj['headers'])
        self.assertEqual(table._num_columns, expected_data['num_columns'])
        self.assertEqual(table._column_max, expected_data['columns_max'])

    def test_headers_update(self):
        data_obj = self.get_data()
        expected_data = self.get_test_expected()['test_headers_update']

        table = Table(data_obj['data'], data_obj['headers'])
        table.headers = table.headers[:-2]
        self.assertEqual(table._num_columns, expected_data['num_columns'])
        self.assertEqual(table._column_max, expected_data['columns_max'])

    def test_data_update(self):
        data_obj = self.get_data()
        expected_data = self.get_test_expected()['test_data_update']

        table = Table(data_obj['data'], data_obj['headers'])
        table.table_data = list.reverse(table.table_data)
        self.assertEqual(table._num_columns, expected_data['num_columns'])
        self.assertEqual(table._column_max, expected_data['columns_max'])

    def test_data_addition(self):
        data_obj = self.get_data()
        expected_data = self.get_test_expected()['test_data_addition']

        table = Table(data_obj['data'], data_obj['headers'])
        table.add_data(['This is a testcase', 'Short', 'but it does his job', ';)'])
        self.assertEqual(table._num_columns, expected_data['num_columns'])
        self.assertEqual(table._column_max, expected_data['columns_max'])

    def test_getitem(self):
        data_obj = self.get_data()
        expected_data = self.get_test_expected()['test_getitem']

        table = Table(data_obj['data'], data_obj['headers'])
        self.assertEqual(table[0], expected_data['first_row'])
        self.assertEqual(table[2][1], expected_data['third_row_second_col'])

    def test_no_data_print(self):
        table = Table()
        with self.assertWarns(UserWarning):
            table.print_table()

    def preview(self, table):
        """
        Prints test table data
        """
        table.print_table()
        print("\n** Stats **\n > num_columns: {}\n > columns_size: {}".format(table._num_columns,
                                                                              table._column_max))

    def get_data(self):
        """
        Returns a `dict` with the test data
        """
        if not self._data_dict:
            self._data_dict = self._load_data()

        return self._data_dict

    def get_test_expected(self):
        """
        Returns a `list` with the expected outputs for test
        """
        if not self._expected_data:
            self._expected_data = self._load_test_expected()

        return self._expected_data

    def _load_data(self):

        # Load data
        input_data_path = self._input_folder / 'table_data.json'
        with open(input_data_path.resolve(), 'r') as f:
            data_obj = json.load(f)

        # Reduce data size
        line_lenght = len(data_obj['headers'])
        step = 60
        num_lines = 14
        small_data = []
        for i in range(num_lines):
            small_data.extend(data_obj['data'][i * step:i * step + line_lenght])

        data_obj['data'] = small_data
        return data_obj

    def _load_test_expected(self):
        expected_file_path = self._input_folder / 'test_tab_expected.json'
        with open(expected_file_path.resolve(), 'r') as f:
            expected_data = json.load(f)

        return expected_data


if __name__ == '__main__':
    TestTable().preview()



# -*- encoding: utf-8 -*-
from setuptools import setup, find_packages

with open('README.md') as f:
    readme = f.read()

setup(
    name='tablat',
    version='0.1.1',
    description='Print basic tables in python',
    long_description=readme,
    long_description_content_type='text/markdown',
    author='salpreh',
    author_email='salva.perez46@gmail.com',
    url='https://github.com/salpreh/tablat',
    license='MIT License',
    packages=find_packages(exclude=('test', 'assets', 'venv', 'doc'))
)
